package com.nisum.employee.controller;

import com.nisum.employee.dto.EmployeeDto;
import com.nisum.employee.dto.EmployeeWithDepartment;
import com.nisum.employee.entities.Employee;
import com.nisum.employee.service.EmployeeService;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.rmi.NoSuchObjectException;
import java.util.Optional;

@RestController
@RequestMapping("/api/v1")
@Slf4j
public class EmployeeController {

    @Autowired
    private EmployeeService service;


    /**
     * Creates new employee into the employee db and return the created object.
     *
     * @param employee employee request body
     * @return response body of employee object
     */
    @PostMapping("/employees")
    public ResponseEntity<EmployeeDto> saveEmployee(@Valid @RequestBody(required = false)  EmployeeDto employee) {
        log.info("Employee is created");
        return ResponseEntity.ok(service.saveEmployee(employee));
    }

    /**
     * Gets an employee based on the employee id input.
     *
     * @param id employee id
     * @return response employee
     */
    @GetMapping("/emp/{id}")
    public Employee findEmployee(@PathVariable("id") Long id){
        log.info("find method of Employee Controller class");
        return  service.findEmployee(id);
    }

    /**
     * Gets an employee with Department based on the employee id input.
     *
     * @param empId employee empId
     * @return response employee with Department Object
     */
    @GetMapping("/{id}")
    public ResponseEntity<EmployeeWithDepartment> getEmployee(@NonNull @PathVariable("id") long empId){
        log.info("Employee get method");
        EmployeeWithDepartment withDepartment = service.getEmployee(empId);
        return new ResponseEntity<EmployeeWithDepartment>(withDepartment, HttpStatus.OK);

    }

    /**
     * Reading Request Headers
     * @param language
     * @return
     */

    @GetMapping("/employeeMessage")
    public ResponseEntity<String> greeting(@RequestHeader(HttpHeaders.ACCEPT_LANGUAGE) String language) {
        // code that uses the language variable
        return new ResponseEntity<String>("", HttpStatus.OK);
    }
}
