package com.nisum.employee.exception;

import com.nisum.employee.dto.ApiError;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import javax.persistence.EntityNotFoundException;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestControllerAdvice
public class ControllerAdviceException extends ResponseEntityExceptionHandler {

    @ExceptionHandler(NoDataFoundException.class)
    public ResponseEntity<Object> handleNoDataException(NoDataFoundException nde, WebRequest request){
        Map<String, Object> map = new LinkedHashMap<>();
        map.put("message","No Data found");
        map.put("timestamp", LocalDateTime.now());

        return new ResponseEntity<>(map, NOT_FOUND);
    }

    @ExceptionHandler(value = EmployeeAlreadyExistsException.class)
    public ResponseEntity<Object> handleEmployeeAlreadyExistsException(EmployeeAlreadyExistsException empAlreadyExistsException) {
        return new ResponseEntity<Object>("Employee already exists", HttpStatus.CONFLICT);
    }

    /**
     * If there is no Entity present in the object
     */
    @ExceptionHandler(EntityNotFoundException.class)
    protected ResponseEntity<Object> handleEntityNotFound(EntityNotFoundException ex,WebRequest request) {
        ApiError apiError = new ApiError(NOT_FOUND);
        apiError.setMessage(ex.getMessage());
        return new ResponseEntity<Object>(apiError, NOT_FOUND);
    }

    /**
     * Unable to instantiate bean for this method
     * Need to check and resolve
     */

//    @ExceptionHandler(value = MethodArgumentNotValidException.class)
//    public ResponseEntity<ApiError> handleMethodArgumentNotValid(MethodArgumentNotValidException ex) {
//
//        ApiError errorDetails = new ApiError(BAD_REQUEST);
//        errorDetails.setMessage(ex.getMessage());
//        errorDetails.setTimestamp(LocalDateTime.now());
//        return new ResponseEntity<ApiError>(errorDetails, HttpStatus.BAD_REQUEST);
//    }
}
