package com.nisum.employee.service;

import com.nisum.employee.dto.DepartmentDto;
import com.nisum.employee.dto.EmployeeDto;
import com.nisum.employee.dto.EmployeeWithDepartment;
import com.nisum.employee.entities.Employee;
import com.nisum.employee.exception.EmployeeAlreadyExistsException;
import com.nisum.employee.exception.NoDataFoundException;
import com.nisum.employee.repository.EmployeeRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.persistence.EntityNotFoundException;
import java.util.Optional;

@Service
@Slf4j
public class EmployeeService {

    @Autowired
    private EmployeeRepository repository;

    @Autowired
    private RestTemplate restTemplate;

    /**
     * savng employee from employee object
     * @param employee
     * @return
     */
    public EmployeeDto saveEmployee(EmployeeDto employee) {
        Optional.of(employee).orElseThrow(()->new NoDataFoundException(employee.getEmployeeId()));
        Employee emp = EmployeeMapperService.MAPPER.employeeDtoToEntity(employee);
        log.info("save method of Employee Service class");

        if (repository.existsById(emp.getEmployeeId())) {
            throw new EmployeeAlreadyExistsException();
        }
        Employee saved = repository.save(emp);
        return EmployeeMapperService.MAPPER.employeeEntityToDto(saved);
    }

    public EmployeeWithDepartment getEmployee(long employeeId) {
        log.info("get method of Employee Service class");
        EmployeeWithDepartment empDep = new EmployeeWithDepartment();
        //empId
        //db
        Employee employee = Optional.ofNullable(repository.findByEmployeeId(employeeId)).orElseThrow(() -> new EntityNotFoundException());
        EmployeeDto employeeDto = EmployeeMapperService.MAPPER.employeeEntityToDto(employee);

        //department not present
        DepartmentDto department = restTemplate.getForObject("http://localhost:9001/department/" + employee.getEmpDepartmentId(), DepartmentDto.class);

        empDep.setEmployee(employeeDto);
        empDep.setDepartment(department);

        return empDep;
    }

    public Employee findEmployee(long id){
        log.info("find method of Employee Service class");
        return repository.findById(id).orElseThrow(()-> new NoDataFoundException(id));
    }



}
